﻿using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Enums.KnowledgeBase;
using Hotline.Users;
using MapsterMapper;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.KnowledgeBase
{
    public class KnowledgeDomainService : IKnowledgeDomainService, IScopeDependency
    {
        private readonly IUserRepository _userRepository;
        private readonly IKnowledgeRepository _knowledgeRepository;
        private readonly IKnowledgeTempRepository _knowledgeTempRepository;
        private readonly IMapper _mapper;
        private readonly IKnowledgePVRepository _knowledgePVRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userRepository"></param>
        /// <param name="knowledgeRepository"></param>
        /// <param name="knowledgeTempRepository"></param>
        /// <param name="mapper"></param>
        /// <param name="knowledgePVRepository"></param>
        public KnowledgeDomainService(IUserRepository userRepository, IKnowledgeRepository knowledgeRepository, IKnowledgeTempRepository knowledgeTempRepository, IMapper mapper, IKnowledgePVRepository knowledgePVRepository)
        {
            _userRepository = userRepository;
            _knowledgeRepository = knowledgeRepository;
            _mapper = mapper;
            _knowledgeTempRepository = knowledgeTempRepository;
            _knowledgePVRepository = knowledgePVRepository;
        }

        /// <summary>
        /// 根据用户ID获取当前用户所属部门及下级部门Id
        /// </summary>
        /// <param name="RequiredOrgId"></param>
        /// <returns></returns>
        public async Task<List<string>> GetChildBMId(string? RequiredOrgId)
        {
            List<string> list = new();
            if (!string.IsNullOrEmpty(RequiredOrgId))
            {
                //根据父级获取子级信息
                var tempList = await _knowledgeRepository.GetChild(RequiredOrgId);
                //获取子级ID
                list = tempList.Select(it => it.Id).ToList();
            }
            return list;
        }

        /// <summary>
        /// 知识库-知识修改-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<Knowledge> KnowledgeUpdateInfo(string Id, CancellationToken cancellationToken)
        {
            //先查询正式数据，如果正式数据不存在在查询草稿箱
            var know = await _knowledgeRepository.GetAsync(Id, cancellationToken);
            if (know is null)
            {
                //查询草稿箱数据
                var tempKnow = await _knowledgeTempRepository.GetAsync(Id, cancellationToken);
                if (tempKnow != null)
                {
                    //草稿数据处理成通用返回数据
                    know = new Knowledge();
                    _mapper.Map(tempKnow, know);
                }
            }
            return know;
        }

        /// <summary>
        /// 知识库-查询详情-增加浏览量
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<Knowledge> KnowledgeInfo(string Id, CancellationToken cancellationToken)
        {
            var know = await _knowledgeRepository.GetAsync(Id, cancellationToken);
            if (know is null)
                throw UserFriendlyException.SameMessage("知识查询失败！");
            //写入浏览记录
            KnowledgePv knowledgePv = new()
            {
                CreationTime = DateTime.Now,
                KnowledgeId = know.Id
            };
            //浏览记录写入
            await _knowledgePVRepository.AddAsync(knowledgePv, cancellationToken);

            //修改浏览量
            know.PageView++;
            //修改点击量
            await _knowledgeRepository.UpdateAsync(know, cancellationToken);
            return know;
        }

        /// <summary>
        /// 知识库-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<string> KnowledgeAdd(AddKnowledgeDto dto, CancellationToken cancellationToken)
        {
            var kn = _mapper.Map<Knowledge>(dto);
            //知识草稿ID如果不为空，则表示是草稿箱的数据
            if (!string.IsNullOrEmpty(dto.Code))
            {
                //移除草稿箱数据
                await _knowledgeTempRepository.RemoveAsync(p => p.Code == dto.Code, false, cancellationToken);
            }
            //如果草稿箱不存在数据，也就是Code为空，从新生成Code
            if (string.IsNullOrEmpty(kn.Code))
                kn.Code = Convert.ToInt64((DateTime.Now - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalSeconds).ToString();
            kn.PageView = 0;
            kn.Status = EKnowledgeStatus.Auditing;

            return await _knowledgeRepository.AddAsync(kn, cancellationToken);
            //调用流程创建

            //如果流程创建失败需要删除此条数据
            //await _knowledgeRepository.RemoveAsync(knowledge);
        }

        /// <summary>
        /// 知识库-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task KnowledgeUpdate(UpdateKnowledgeDto dto, CancellationToken cancellationToken)
        {
            //知识修改逻辑
            //如果可以修改，此条数据不动，重新写入一条新数据，知识编号不变，版本号+1

            var know = await _knowledgeRepository.GetAsync(dto.Id, cancellationToken);
            if (know is null || know.IsDeleted)
                throw UserFriendlyException.SameMessage("无效知识");
            //审批中数据不能处理
            if (know.Status == EKnowledgeStatus.Auditing)
                throw UserFriendlyException.SameMessage("审批中数据不能修改");
            var knowledge = _mapper.Map<Knowledge>(dto);
            knowledge.Code = know.Code;
            knowledge.PageView = 0;
            knowledge.Status = EKnowledgeStatus.Auditing;
            knowledge.Id = "";
            knowledge.Version = know.Version + 1;
            //新增一条数据
            await _knowledgeRepository.AddAsync(knowledge, cancellationToken);

            //调用流程创建

            //如果流程创建失败需要删除此条数据
            //await _knowledgeRepository.RemoveAsync(knowledge);
        }

        /// <summary>
        /// 审批操作
        /// </summary>
        /// <param name="Ids">需要审批的数据ID</param>
        /// <param name="ApprovalStatus">审核状态：0：通过；1：不通过</param>
        /// <param name="HandleOpinions">办理意见</param>
        /// <returns></returns>
        public async Task ApprovalHandle(List<string> Ids, string ApprovalStatus, string HandleOpinions, CancellationToken cancellationToken)
        {
            //审批操作
            foreach (var item in Ids)
            {
                var know = await _knowledgeRepository.GetAsync(item, cancellationToken);
                if (know != null)
                {
                    //调用流程审批接口
                    //流程调用成功以后在处理以下业务

                    //处理数据
                    if (ApprovalStatus == "0")
                    {
                        know.Status = EKnowledgeStatus.Succeed;
                        know.IsOnShelf = true;
                        know.OnShelfTime = DateTime.Now;
                    }
                    else
                    {
                        know.Status = EKnowledgeStatus.Failed;
                    }
                    await _knowledgeRepository.UpdateAsync(know, cancellationToken);
                }
            }
        }
    }
}

